@@ -18,244 +18,191 @@ package Lemonldap::NG::Handler::SharedConf;
 #use strict;
 
 use Lemonldap::NG::Handler::Main qw(:all);
-use Lemonldap::NG::Handler::Initialization::GlobalInit;
-
-#use Lemonldap::NG::Handler::Vhost;
+use Lemonldap::NG::Handler::Main::Logger;
+use Lemonldap::NG::Handler::API qw(:httpCodes);
+use Lemonldap::NG::Handler::Reload;
 use Lemonldap::NG::Common::Conf;               #link protected lmConf
 use Lemonldap::NG::Common::Conf::Constants;    #inherits
-use Cache::Cache qw($EXPIRES_NEVER);
-use Lemonldap::NG::Handler::Main::Logger;
 
-use Mouse;
+use base qw(Lemonldap::NG::Handler::Main);
 
-extends qw(Lemonldap::NG::Handler::Main);      # Lemonldap::NG::Handler::Vhost
-
-#use base qw(Lemonldap::NG::Handler::Vhost Lemonldap::NG::Handler::Main);
-
-#parameter reloadTime Time in second between 2 configuration check (600)
-
-our $VERSION    = '1.4.0';
-our $cfgNum     = 0;
-our $lastReload = 0;
-our $reloadTime;
-our $lmConf;
-our $localConfig;
+our $VERSION = '1.9.1';
+our $lmConf;         # Lemonldap::NG::Common::Conf object to get config
+our $localConfig;    # Local configuration parameters, i.e. defined
+                     # in lemonldap-ng.ini or in startup parameters
+our $cfgNum    = 0;      # Number of the loaded remote configuration
+our $lastCheck = 0;      # Date of last configuration check (unix time)
+our $checkTime = 600;    # Time between 2 configuration check (in seconds);
+                         # default value is 600, can be reset in local config
 
 BEGIN {
-    if ( MP() == 2 ) {
-        eval {
-            require threads::shared;
-            Apache2::RequestUtil->import();
-            threads::shared::share($cfgNum);
-            threads::shared::share($lastReload);
-            threads::shared::share($reloadTime);
-            threads::shared::share($lmConf);
-            threads::shared::share($localConfig);
-        };
-    }
+    Lemonldap::NG::Handler::API->thread_share($cfgNum);
+    Lemonldap::NG::Handler::API->thread_share($lastCheck);
+    Lemonldap::NG::Handler::API->thread_share($checkTime);
+    Lemonldap::NG::Handler::API->thread_share($lmConf);
+    Lemonldap::NG::Handler::API->thread_share($localConfig);
     *EXPORT_TAGS = *Lemonldap::NG::Handler::Main::EXPORT_TAGS;
     *EXPORT_OK   = *Lemonldap::NG::Handler::Main::EXPORT_OK;
     push(
         @{ $EXPORT_TAGS{$_} },
-        qw($cfgNum $lastReload $reloadTime $lmConf $localConfig)
+        qw($cfgNum $lastCheck $checkTime $lmConf $localConfig)
     ) foreach (qw(variables localStorage));
-    push @EXPORT_OK, qw($cfgNum $lastReload $reloadTime $lmConf $localConfig);
+    push @EXPORT_OK, qw($cfgNum $lastCheck $checkTime $lmConf $localConfig);
 }
 
 # INIT PROCESS
 
 ## @imethod void init(hashRef args)
-# Constructor.
-# init is overloaded to call only localInit. globalInit is called later.
+# Read parameters and build the Lemonldap::NG::Common::Conf object.
 # @param $args hash containing parameters
 sub init($$) {
-    my ( $class, $args ) = splice @_;
+    my ( $class, $args ) = @_;
 
-    # TODO reloadTime in defaultValuesInit ?
-    $reloadTime = $args->{reloadTime} || 600;
-    $class->localInit($args);
-}
-
-## @imethod protected void defaultValuesInit(hashRef args)
-# Set default values for non-customized variables
-# @param $args hash containing parameters
-# @return boolean
-sub defaultValuesInit {
-    my ( $class, $args ) = splice @_;
-
-    # Local configuration overrides global configuration
-    my %h = ( %$args, %$localConfig );
-
-    #return $class->SUPER::defaultValuesInit( \%h );
-
-    my $globalinit = Lemonldap::NG::Handler::Initialization::GlobalInit->new(
-        customFunctions => $tsv->{customFunctions},
-        useSafeJail     => $tsv->{useSafeJail},
-    );
-
-    (
-        @$tsv{
-            qw( cookieName      securedCookie      whatToTrace
-              https           port               customFunctions
-              timeoutActivity useRedirectOnError useRedirectOnForbidden
-              useSafeJail     key                maintenance )
-        },
-        @$ntsv{
-            qw( cda             httpOnly           cookieExpiration
-              cipher
-              )
-        }
-      )
-      = $globalinit->defaultValuesInit(
-        @$tsv{
-            qw( cookieName      securedCookie      whatToTrace
-              https           port               customFunctions
-              timeoutActivity useRedirectOnError useRedirectOnForbidden
-              useSafeJail     key                maintenance )
-        },
-        @$ntsv{
-            qw( cda             httpOnly           cookieExpiration
-              cipher )
-        },
-        \%h
-      );
-
-}
-
-## @imethod void localInit(hashRef args)
-# Load parameters and build the Lemonldap::NG::Common::Conf object.
-# @return boolean
-sub localInit {
-    my ( $class, $args ) = splice @_;
-    die(
-"$class : unable to build configuration : $Lemonldap::NG::Common::Conf::msg"
-      )
-      unless ( $lmConf =
-        Lemonldap::NG::Common::Conf->new( $args->{configStorage} ) );
-
-    # Get local configuration parameters
-    my $localconf = $lmConf->getLocalConf(HANDLERSECTION);
-    if ($localconf) {
-        $args->{$_} ||= $localconf->{$_} foreach ( keys %$localconf );
-    }
-
-    # Store in localConfig global variable
-    $localConfig = $args;
-
-    # localStorage can be declared in configStorage or at the root or both
+    # According to doc, localStorage can be declared in $args root,
+    # but it must be in $args->{configStorage}
     foreach (qw(localStorage localStorageOptions)) {
-        $args->{$_} ||= $args->{configStorage}->{$_} || $lmConf->{$_};
         $args->{configStorage}->{$_} ||= $args->{$_};
     }
 
-    $class->defaultValuesInit($args);
-    $class->SUPER::localInit($args);
+    $lmConf = Lemonldap::NG::Common::Conf->new( $args->{configStorage} );
+    die(    "$class : unable to build configuration: "
+          . "$Lemonldap::NG::Common::Conf::msg" )
+      unless ($lmConf);
+
+    # Merge local configuration parameters so that params defined in
+    # startup parameters have precedence over lemonldap-ng.ini params
+    $localConfig = { %{ $lmConf->getLocalConf(HANDLERSECTION) }, %{$args} };
+
+    $checkTime = $localConfig->{checkTime} || $checkTime;
+
+    # Few actions that must be done at server startup:
+    # * set log level for Lemonldap::NG logs
+    Lemonldap::NG::Handler::Main::Logger->logLevelInit(
+        $localConfig->{logLevel} );
+
+    # * set server signature
+    $class->serverSignatureInit unless ( $localConfig->{hideSignature} );
+
+    # * launch status process
+    $class->statusInit($tsv) if ( $localConfig->{status} );
+    1;
+}
+
+# @method void serverSignatureInit
+# adapt server signature
+sub serverSignatureInit {
+    my $class = shift;
+    Lemonldap::NG::Handler::API->setServerSignature(
+        "Lemonldap::NG/" . $Lemonldap::NG::Handler::VERSION )
+      if ($Lemonldap::NG::Handler::VERSION);
+}
+
+## @ifn protected void statusInit()
+# Launch the status process
+sub statusInit {
+    my ( $class, $tsv ) = @_;
+    return if ( $tsv->{statusPipe} and $tsv->{statusOut} );
+    require IO::Pipe;
+    $statusPipe = IO::Pipe->new;
+    $statusOut  = IO::Pipe->new;
+    if ( my $pid = fork() ) {
+
+        # TODO: log new process pid
+        $statusPipe->writer();
+        $statusOut->reader();
+        $statusPipe->autoflush(1);
+        ( $tsv->{statusPipe}, $tsv->{statusOut} ) = ( $statusPipe, $statusOut );
+    }
+    else {
+        $statusPipe->reader();
+        $statusOut->writer();
+        my $fdin  = $statusPipe->fileno;
+        my $fdout = $statusOut->fileno;
+        open STDIN,  "<&$fdin";
+        open STDOUT, ">&$fdout";
+        my $perl_exec = ( $^X =~ /perl/ ) ? $^X : 'perl';
+        exec $perl_exec, '-MLemonldap::NG::Handler::Status',
+          map( {"-I$_"} @INC ),
+          '-e &Lemonldap::NG::Handler::Status::run()';
+    }
 }
 
 # MAIN
 
-## @rmethod int run(Apache2::RequestRec r)
+## @rmethod int run
 # Check configuration and launch Lemonldap::NG::Handler::Main::run().
-# Each $reloadTime, the Apache child verify if its configuration is the same
+# Each $checkTime, the Apache child verify if its configuration is the same
 # as the configuration stored in the local storage.
-# @param $r Apache2::RequestRec object
+# @param $rule optional Perl expression to grant access
 # @return Apache constant
-sub run($$) {
-    my ( $class, $r ) = splice @_;
-    if ( time() - $lastReload > $reloadTime ) {
+sub run {
+    my $class = shift;
+    if ( time() - $lastCheck > $checkTime ) {
         die("$class: No configuration found")
-          unless ( $class->testConf(1) == OK );
+          unless ( $class->checkConf );
     }
-    return $class->SUPER::run($r);
+    if ( my $rule = shift ) {
+        my ( $cond, $prot ) =
+          Lemonldap::NG::Handler::Reload->conditionSub( $rule, $tsv );
+        return $class->SUPER::run( $cond, $prot );
+    }
+    return $class->SUPER::run();
 }
 
 # CONFIGURATION UPDATE
 
-## @rmethod protected int testConf(boolean local)
-# Test if configuration has changed and launch setConf() if needed.
-# If the optional boolean $local is true, remote configuration is not tested:
-# only local cached configuration is tested if available. $local is given to
-# Lemonldap::NG::Common::getConf()
-# @param $local boolean
-# @return Apache constant
-sub testConf {
-    my ( $class, $local ) = splice @_;
-    my $conf = $lmConf->getConf( { local => $local } );
+## @rmethod protected int checkConf(boolean force)
+# Check if configuration is up to date, and reload it if needed.
+# If the optional boolean $force is set to true,
+# * cached configuration is ignored
+# * and checkConf returns false if it fails to load remote config
+# @param $force boolean
+# @return true if config is up to date or if reload config succeeded
+sub checkConf {
+    my ( $class, $force ) = @_;
+    my $conf = $lmConf->getConf( { local => !$force } );
+
     unless ( ref($conf) ) {
         Lemonldap::NG::Handler::Main::Logger->lmLog(
 "$class: Unable to load configuration: $Lemonldap::NG::Common::Conf::msg",
             'error'
         );
-        return $cfgNum ? OK : SERVER_ERROR;
+        return $force ? 0 : $cfgNum ? 1 : 0;
     }
+
     if ( !$cfgNum or $cfgNum != $conf->{cfgNum} ) {
         Lemonldap::NG::Handler::Main::Logger->lmLog(
-"$class: get configuration $conf->{cfgNum} ($Lemonldap::NG::Common::Conf::msg)",
+"Get configuration $conf->{cfgNum} ($Lemonldap::NG::Common::Conf::msg)",
             'debug'
         );
-        $lastReload = time();
-        return $class->setConf($conf);
+        $lastCheck = time();
+        unless ( $cfgNum = $conf->{cfgNum} ) {
+            Lemonldap::NG::Handler::Main::Logger->lmLog(
+                'No configuration available', 'error' );
+            return 0;
+        }
+        $conf->{$_} = $localConfig->{$_} foreach ( keys %$localConfig );
+        Lemonldap::NG::Handler::Reload->configReload( $conf, $tsv );
     }
     Lemonldap::NG::Handler::Main::Logger->lmLog(
         "$class: configuration is up to date", 'debug' );
-    OK;
-}
-
-## @rmethod protected int setConf(hashRef conf)
-# Launch globalInit().
-# Local parameters have best precedence on configuration parameters.
-# @return Apache constant
-sub setConf {
-    my ( $class, $conf ) = splice @_;
-
-    # Local configuration overrides global configuration
-    $cfgNum = $conf->{cfgNum};
-    $conf->{$_} = $localConfig->{$_} foreach ( keys %$localConfig );
-    $class->globalInit($conf);
-    OK;
+    return 1;
 }
 
 # RELOAD SYSTEM
 
-*reload = *refresh;
+*refresh = *reload;
 
-## @rmethod int refresh(Apache::RequestRec r)
-# Launch testConf() with $local=0, so remote configuration is tested.
+## @rmethod int reload
+# Launch checkConf() with $local=0, so remote configuration is tested.
 # Then build a simple HTTP response that just returns "200 OK" or
 # "500 Server Error".
-# @param $r current request
 # @return Apache constant (OK or SERVER_ERROR)
-sub refresh($$) {
-    my ( $class, $r ) = splice @_;
+sub reload {
+    my $class = shift;
     Lemonldap::NG::Handler::Main::Logger->lmLog(
-        "$class: request for configuration reload", 'notice' );
-    $r->handler("perl-script");
-    if ( $class->testConf(0) == OK ) {
-        if ( MP() == 2 ) {
-            $r->push_handlers( 'PerlResponseHandler' =>
-                  sub { my $r = shift; $r->content_type('text/plain'); OK } );
-        }
-        elsif ( MP() == 1 ) {
-            $r->push_handlers(
-                'PerlHandler' => sub { my $r = shift; $r->send_http_header; OK }
-            );
-        }
-        else {
-            return 1;
-        }
-    }
-    else {
-        if ( MP() == 2 ) {
-            $r->push_handlers( 'PerlResponseHandler' => sub { SERVER_ERROR } );
-        }
-        elsif ( MP() == 1 ) {
-            $r->push_handlers( 'PerlHandler' => sub { SERVER_ERROR } );
-        }
-        else {
-            return 0;
-        }
-    }
-    return OK;
+        "Request for configuration reload", 'notice' );
+    return $class->checkConf(1) ? DONE : SERVER_ERROR;
 }
 
 1;
@@ -327,7 +274,7 @@ the database.
 
 Like L<Lemonldap::NG::Handler::Main>::init() but read only localStorage
 related options. You may change default time between two configuration checks
-with the C<reloadTime> parameter (default 600s).
+with the C<checkTime> parameter (default 600s).
 
 =head1 OPERATION
 
@@ -353,7 +300,7 @@ L<http://lemonldap-ng.org/>
 
 =item Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
 
-=item Franois-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
+=item François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
 
 =item Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
@@ -373,11 +320,11 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =over
 
-=item Copyright (C) 2006, 2007, 2008, 2009, 2010, 2013 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
+=item Copyright (C) 2005-2016 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
-=item Copyright (C) 2012 by Franois-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
+=item Copyright (C) 2012 by François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
 
-=item Copyright (C) 2006, 2008, 2009, 2010, 2011, 2012 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
+=item Copyright (C) 2006-2016 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
 
 =back
 
