@@ -7,7 +7,7 @@ package Lemonldap::NG::Portal::AuthMulti;
 
 use Lemonldap::NG::Portal::_Multi;    #inherits
 
-our $VERSION = '1.4.6';
+our $VERSION = '1.9.5';
 
 sub authInit {
     my $self = shift;
@@ -36,6 +36,11 @@ sub authFinish {
     return $self->_multi->try( 'authFinish', 0 );
 }
 
+sub authPostStore {
+    my $self = shift;
+    return $self->_multi->try( 'authPostStore', 0 );
+}
+
 sub authLogout {
     my $self = shift;
     return $self->_multi->try( 'authLogout', 0 );
@@ -43,7 +48,9 @@ sub authLogout {
 
 sub authForce {
     my $self = shift;
-    return $self->_multi->try( 'authForce', 0 );
+    $self->_sub('authInit');
+    my $s    = $self->{_multi}->{last}->[0] . "::authForce";
+    return &{$s}($self);
 }
 
 sub getDisplayType {
