@@ -16,7 +16,7 @@ use Tie::File;
 my $offline_script_url =
   'http://lemonldap-ng.org/lib/plugins/offline/create.php';
 my $offline_zip_url = 'http://lemonldap-ng.org/lib/plugins/offline/offline.zip';
-my $remove_versions = [qw/latest 1.0 1.1 1.2 1.3 2.0/];
+my $remove_versions = [qw/latest 1.0 1.1 1.2 1.3 1.4 2.0/];
 my $rc;
 
 # Launch remote offline script
@@ -46,7 +46,7 @@ system('find . -name \*.html -exec sed -i "s#/latest/#/current/#g" {} \;');
 tie my @documentation, 'Tie::File', 'pages/documentation.html'
   or die("Unable to open pages/documentation.html");
 
-splice @documentation, 61, 24;
+splice @documentation, 61, 27;
 
 # Correct some bad media links
 system('find . -name \*.html -exec sed -i "s#/_media#../media#g" {} \;');
@@ -66,10 +66,23 @@ my @links = (
 
 splice @index, 21, 2, @links;
 
-# Remove external logos
+# Remove external logos and replace them by a default image
 system('rm -rf media/applications/ media/logos/');
+system('find . -name \*.html -exec sed -i "s#/media/applications/[^\.]*\.\(png\|jpg\|gif\)#/media/icons/kmultiple.png#g" {} \;');
+system('find . -name \*.html -exec sed -i "s#/media/logos/[^\.]*\.\(png\|jpg\|gif\)#/media/icons/kmultiple.png#g" {} \;');
 
 # Remove Facebook iframe (See http://jira.ow2.org/browse/LEMONLDAP-674)
 system('sed -i "/<iframe/d" pages/contact.html');
 
+# Remove background image
+system('sed -i "s#/lib/tpl/bootstrap3/images/background.jpg##g" css/*');
+
+# Download missing images
+system('mkdir -p lib/bootstrap3/images && cd lib/bootstrap3/images && wget -l 1 -nd -r http://lemonldap-ng.org/lib/tpl/bootstrap3/images/ && rm -f index* && cd -');
+
+# Link duplicates files
+system(q@find . -type f -printf "%s %p\n"|sort -n|perl -a -ne 'if($F[0]==$lf&&`md5sum $F[1]|cut -f1 -d" "` eq `md5sum $ln|cut -f1 -d" "`){print "ln -srf $ln $F[1]\n"}else{$lf=$F[0];$ln=$F[1]}'|sh@);
+
+system("find lib/bootstrap3/images -name '*.1' -delete");
+
 exit 0;
